/**********************************************************************

triang.h
Mark Cederholm (plp@pierssen.com)
Version 1.0

Class Triang will read a single polygon record from a shapefile, check
it for self-touching, triangulate it, and write the resulting triangles
to an output shapefile.  See "seidel.h" for more info on the
triangulation algorithm.  The shapefile I/O routines are quick and
dirty, and should not be used as a guide for creating a general
shapefile class.  The polygon record to be triangulated MUST be clean
and have no redundant vertices.

Method do_it has the following return values:

0 = successful
1 = error reading shapefile
2 = polygon has redundant vertices
3 = error in triangulation
4 = error writing shapefile

**********************************************************************/

#include "seidel.h"
#include <stdio.h>
#include <string.h>

class Triang
{

   public:

   Triang(){};
   int do_it(char * a_shp_in, char * a_shp_out);

   private:

   // data structures

   char * shp_in;
   char * shp_out;
   int ncontours;
   int ntriangles;
   int nvertices;
   int * cntr;
   int * c;
   int * islands;
   int (* t)[3];
   double (* vertices)[2];
   double (* v)[2];
   double (* triangles)[6];
   double bounds[4];

   // methods

   int read_segments();
   bool has_redundant_vertices();
   int gen_triangles();
   bool write_triangles();
   double get_ring_area(int i0, int n);
   int build_ring_array(int ring_id, bool add_islands);
   void write_sh_header(FILE * out_file, size_t fsize);
   void write_db_header(FILE * out_file, int numrecs);
   unsigned long endswap(unsigned long n);

};

